'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var url = _interopDefault(require('url'));
var lodash = require('lodash');

function isUnparseableQuery() {
  var query = arguments.length <= 0 || arguments[0] === undefined ? 'key=value' : arguments[0];

  return lodash.isError(lodash.attempt(decodeURIComponent, query));
}

function validateQueryParams() {
  var queries = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];

  return !lodash.some(queries, isUnparseableQuery);
}

function dropInvalidQueryParams() {
  var queries = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];

  return lodash.reject(queries, isUnparseableQuery);
}

var QUERY_DELIMITER = '&';

var defaultRedirect = function defaultRedirect(req, res, next) {
  var context = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

  var nextPath = req.path + '?' + context.nextQuery.join(context.delimiter);
  console.error('\nInvalid query detected. Dropping unparseable params:\n' + '\t' + context.droppedParams.join('\n\t') + '\nand redirecting to:\n' + '\t' + nextPath + '\n');

  res.redirect(nextPath);
  return;
};

function queryValidator() {
  var cb = arguments.length <= 0 || arguments[0] === undefined ? defaultRedirect : arguments[0];
  var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var delimiter = options.delimiter || QUERY_DELIMITER;

  return function middleware(req, res, next) {
    var raw = url.parse(req.url, false).query || '';
    var oldQuery = raw.split(delimiter);

    if (!raw || validateQueryParams(oldQuery)) {
      next();
      return;
    }

    var nextQuery = dropInvalidQueryParams(oldQuery);
    var droppedParams = lodash.difference(oldQuery, nextQuery);
    var error = new Error('Invalid query param(s) detected');

    var context = {
      delimiter: delimiter,
      error: error,
      oldQuery: oldQuery,
      nextQuery: nextQuery,
      droppedParams: droppedParams
    };

    cb(req, res, next, context);
    return;
  };
}

module.exports = queryValidator;
//# sourceMappingURL=queryValidator.js.map
